import 'dart:io';
import 'zapt_sdk_flutter_platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'dart:async';
import 'dart:convert';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:flutter/services.dart';
import 'zapt_webview_flutter.dart';

class ZaptSdkFlutter {
  Future<String?> getMapLink(Map<String, dynamic> arguments) {
    return ZaptSdkFlutterPlatform.instance.getMapLink(arguments);
  }

  Future<void> requestPermissions() {
    return ZaptSdkFlutterPlatform.instance.requestPermissions();
  }

  Future<void> requestPermissionsBackground(Map<String, String> arguments) {
    return ZaptSdkFlutterPlatform.instance
        .requestPermissionsBackground(arguments);
  }

  Future<void> initialize(String placeId) {
    return ZaptSdkFlutterPlatform.instance.initialize(placeId);
  }

  Future<void> setDisableSyncingForPositioning(bool disable) {
    return ZaptSdkFlutterPlatform.instance
        .setDisableSyncingForPositioning(disable);
  }

  Future<void> setDisableSyncingForAnalytics(bool disable) {
    return ZaptSdkFlutterPlatform.instance
        .setDisableSyncingForAnalytics(disable);
  }

  Future<void> addListener(String eventName) {
    return ZaptSdkFlutterPlatform.instance.addListener(eventName);
  }

  Future<void> removeLocationListener() {
    return ZaptSdkFlutterPlatform.instance.removeLocationListener();
  }
}

class ZaptMap extends StatefulWidget {
  final String placeId;
  final Map<String, String>? options;
  const ZaptMap({
    Key? key,
    required this.placeId,
    this.options,
  }) : super(key: key);

  @override
  State<ZaptMap> createState() => _ZaptMapState();
}

class _ZaptMapState extends State<ZaptMap> {
  final _zaptSdkFlutterPlugin = ZaptSdkFlutter();
  ZaptWebViewController? _zaptWebViewController;
  WebViewController? _webViewController;
  String _mapLink = '';
  static const stream = EventChannel("ReactNativeZaptSdkBeaconsFound");

  @override
  void initState() {
    super.initState();
    initializeMap();
    initListemBeacons();
  }

  @override
  void didUpdateWidget(ZaptMap oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.placeId != widget.placeId ||
        !mapEquals(widget.options, oldWidget.options)) {
      initializeMap();
    }
  }

  Future<void> initializeMap() async {
    String mapLink;
    try {
      await _zaptSdkFlutterPlugin.initialize(widget.placeId);
      mapLink = await _zaptSdkFlutterPlugin.getMapLink({
            'placeId': widget.placeId,
            'options': widget.options,
          }) ??
          "";
    } on PlatformException {
      mapLink = '';
    }
    if (mapLink.isNotEmpty) {
      setState(() {
        _mapLink = mapLink;
      });
      if (Platform.isAndroid) {
        _zaptWebViewController?.loadUrl(mapLink);
      } else if (Platform.isIOS) {
        _webViewController?.loadUrl(mapLink);
      }
    }
  }

  Future<void> initListemBeacons() async {
    await _zaptSdkFlutterPlugin.addListener("ReactNativeZaptSdkBeaconsFound");
    stream.receiveBroadcastStream().listen(_updateBeacons);
  }

  void _updateBeacons(beacons) {
    if (beacons.isNotEmpty) {
      String beaconsJSON = jsonEncode(beacons);
      String jsCode = 'window.onBeaconsFound($beaconsJSON);';
      if(Platform.isAndroid) {
        _zaptWebViewController?.runJavascript(jsCode);
      } else if (Platform.isIOS) {
        _webViewController?.runJavascript(jsCode);
      }
    }
  }

  Widget getWebViewByPlatform() {
    if(Platform.isAndroid) {
      return ZaptWebView(
        initialURL: _mapLink,
        onZaptWebViewCreated: (ZaptWebViewController controller) {
          setState(() {
            _zaptWebViewController = controller;
          });
        },
      );
    }else if (Platform.isIOS) {
      return WebView(
          javascriptMode: JavascriptMode.unrestricted,
          onWebViewCreated: (WebViewController webViewController) {
            _webViewController = webViewController;
          },
          gestureNavigationEnabled: true,
          onPageFinished: (String url) {
            initListemBeacons();
          }
      );
    } else {
      throw UnsupportedError(
        "Package not supported by this platform. Current platform: $Platform._operatingSystem",
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: getWebViewByPlatform()
    );
  }
}
